;;; planner-bbdb.el --- BBDB integration for the Emacs Planner

;; Copyright (C) 2001 John Wiegley
;; Copyright (C) 2003 Sandra Jean Chua

;; Author: John Wiegley <johnw@gnu.org>
;; Maintainer: Sacha Chua <sacha@free.net.ph>
;; Version: 2.42
;; Keywords: planner, gnus
;; URL: http://sacha.free.net.ph/notebook/wiki/PlannerMode.php#bbdb

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; This file allows you to refer to your contacts easily from within
;; a planner page.
;;
;; Example:
;; [[bbdb://Sacha.*Chua][Sacha]] will be linked to the blog, web
;; or net fields of the first matching BBDB record.

;;; Code:

(require 'planner)
(require 'bbdb-com)

(defun planner-bbdb-get-name-from-address (address)
  "Return the name for ADDRESS to be used in links."
  (let* ((addr (mail-extract-address-components address))
         (rec (apply 'bbdb-search-simple addr)))
    (if rec
        (bbdb-record-name rec)
      (or (car addr) (cadr addr)))))

(defvar planner-bbdb-plan-field "plan"
  "Field that contains a planner page associated with this record.")

(defun planner-bbdb-annotation-from-bbdb ()
  "If called from a bbdb buffer, return an annotation.
Suitable for use in `planner-annotation-functions'."
  (when (eq major-mode 'bbdb-mode)
      (or (bbdb-record-getprop
           (bbdb-current-record) 'plan)
          ;; From a BBDB entry with a plan page; use that. Yay!
          (concat "[[bbdb://"
                  (let ((name (bbdb-record-name (bbdb-current-record))))
                    (while (string-match " " name)
                      (replace-match  "." t t name))
                    name)
                  "][" (bbdb-record-name (bbdb-current-record))
                  "]]"))))

(defun planner-bbdb-browse-url (url)
  "If this is a BBDB URL, jump to it."
  (when (string-match "^bbdb://\\(.+\\)" url)
    (bbdb (match-string 1 url) nil)))

(defun planner-bbdb-resolve-url (id)
  "Replace ID with the blog, web or e-mail address of the BBDB record."
  (save-match-data
    (let ((record (car (bbdb-search (bbdb-records) id id id))))
      (or (and record
               (or (bbdb-record-getprop record 'blog)
                   (bbdb-record-getprop record 'web)
                   (car (bbdb-record-net record))))
          ""))))

(add-to-list 'planner-resolve-url-table '("bbdb:" . planner-bbdb-resolve-url))
(add-hook 'planner-annotation-functions 'planner-bbdb-annotation-from-bbdb)
(custom-add-option 'planner-annotation-functions
                   'planner-bbdb-annotation-from-bbdb)
(planner-option-customized 'planner-url-list
                           (append (list "bbdb://") planner-url-list))
(add-hook 'planner-browse-url-functions 'planner-bbdb-browse-url)
(defalias 'planner-get-name-from-address 'planner-bbdb-get-name-from-address)

(provide 'planner-bbdb)

;;; planner-bbdb.el ends here
