;;; planner-diary.el --- Diary integration for the Emacs Planner (planner.el)

;; Copyright (C) 2003 Thomas Gehrlein <Thomas.Gehrlein@t-online.de>

;; Emacs Lisp Archive Entry
;; Filename: planner-diary.el
;; Version: 2.42
;; Keywords: hypermedia
;; Author: Thomas Gehrlein <Thomas.Gehrlein@t-online.de>
;; Maintainer: Thomas Gehrlein <Thomas.Gehrlein@t-online.de>
;; Description: Integrate the Emacs Planner with Calendar and Diary
;; URL: http://sacha.free.net.ph/notebook/emacs/planner/planner-diary.el
;; ChangeLog: Can be requested from the maintainer
;; Compatibility: Emacs20, Emacs21

;; This file is not part of GNU Emacs.

;; This is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2, or (at your option) any later
;; version.
;;
;; This is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
;; MA 02111-1307, USA.

;;; Commentary:
;;
;; Place planner-diary.el in your load path and add this to your .emacs:
;;
;;    (require 'planner-diary)
;;
;; You can use planner-diary in two different ways:
;;
;; 1) Put the following line of Lisp code in you day plan pages to display your
;;    diary entries:
;;
;;    <lisp>(planner-diary-entries-here)</lisp>
;;
;;    You can do this automatically:
;;
;;    (setq planner-day-page-template
;;          (concat "* Tasks\n\n\n* Diary\n\n"
;;                  "<lisp>(planner-diary-entries-here)</lisp>\n\n* Notes"))
;;
;;    When you open a day plan page outside Emacs, you will see the line of
;;    LISP code and not your diary entries.
;;
;; 2) Put the following line in your .emacs:
;;
;;    (planner-insinuate-diary)
;;
;;    In order for this to work, you should customize or set
;;    planner-day-page-template to include a "* Diary" entry:
;;
;;    (setq planner-day-page-template
;;          "* Tasks\n\n\n* Schedule\n\n\n* Diary\n\n\n* Notes")
;;
;;    In planner-mode-map, C-c C-e updates the diary sections.
;;
;; If you want to see your diary entries for more than just 1 day, set
;; `planner-diary-number-of-diary-entries' accordingly.  This should work for
;; either of the 2 approaches.

;;; CALENDAR INTEGRATION
;;
;; planner-diary has changed does not activate calendar integration by default.
;; To get calendar integration add this to your .emacs:
;;
;;    (planner-insinuate-calendar)

;; planner-diary now shows day plans even if no file for this day exists.  The
;; old behavior - show the diary buffer for that day instead - made me terribly
;; nervous, so I changed it.  If you want the old behavior, activate
;; `planner-diary-show-day-plan-or-diary'.  The doc string tells you how to do
;; that.

;;; RANDOM THOUGHTS ABOUT PLANNER, CALENDAR, AND DIARY

;; This is planner-diary, not planner-calendar!  Include only stuff that has to
;; do with diary and planner, leave the calendar stuff to
;; planner-insinuate-calendar.

;; Deal with holidays.

;; There are 3 ways to use planner with calendar and diary:

;; 1) Use calendar for browsing the day plan pages.

;; Cool features: all days with day plan pages are marked.  ">" and "<" move to
;; the next or previous page.  (Ordinary diary entries should not be marked.)
;; Entry point: a command (planner-browse-calendar ??)  What about key-bindings
;; for "n" and "N"?

;; 2) Display the day plan pages when you move in calendar.

;; Add a hook to calendar-move-hook.  Add key-bindings for "n" and "N".

;; 3) Automatically update the diary section in day plan pages.

;; Add a function to planner-goto-hook.  This work independent of the first 2.

;; Thanks to:
;;
;; - Jody Klymak (jklymak AT coas DOT oregonstate DOT edu)
;;   Added some documentation
;;
;; - Sacha Chua (sacha AT free DOT net DOT ph)
;;   Thoughts and ideas

;;; Code:

(require 'planner)
(require 'diary-lib)
(require 'calendar)

(defcustom planner-diary-string "* Diary"
  "*Header for the diary section in a day plan page."
  :type 'string
  :group 'planner)

(defcustom planner-diary-number-of-diary-entries 1
  "*Specifies how many days of diary entries are displayed."
  :type 'number
  :group 'planner)


;;; functions
(defun planner-diary-get-diary-entries (date &optional no-of-days)
  "For DATE, get the NO-OF-DAYS diary entries for DATE and the following days.
DATE is a list (month day year).  NO-OF-DAYS defaults to
`planner-diary-number-of-diary-entries'"
  (save-window-excursion
    (let* ((fancy-diary-buffer "temporary-fancy-diary-buffer")
           (entries)
           (no-of-days (or no-of-days planner-diary-number-of-diary-entries)))
      ;; no messages from list-diary-entries
      (flet ((message (&rest args) (ignore args)))
        (list-diary-entries date no-of-days)
        (switch-to-buffer fancy-diary-buffer)

        ;; return empty string if buffer is empty
        (if (= (point-max) 1)
            (setq entries "")
          (setq entries
                (buffer-substring
                 (if (> no-of-days 1)   ; if more than 1 day, show everything
                     (point-min)
                   ;; remove date and lots of =s if just for 1 day
                   (progn (goto-char (point-min))
                          (search-forward-regexp "^=+$") ; one or more =
                          (1+ (point))))
                 ;; remove final newline
                 (progn (goto-char (point-max))
                        (when (bolp) (backward-char 1))
                        (point)))))
        (kill-buffer fancy-diary-buffer)
        entries))))


;;; Put this is your day pages: <lisp>(planner-diary-entries-here)</lisp>
(defun planner-diary-entries-here ()
  "Display the diary for the current planner day pages.
You might want to use `planner-day-page-template'."
  (planner-diary-get-diary-entries
   (planner-filename-to-calendar-date (emacs-wiki-page-name))))


(defun planner-diary-insert-diary (&optional force)
  "Insert the fancy diary for the day into the day plan file.
If FORCE is non-nil, insert `planner-diary-string' at the beginning of the
buffer if it is not there yet."
  ;; search for "^* Diary$", delete buffer content to the next "^* ", insert
  (interactive "P") ; raw prefix arg
  ;; sanity checks
  (unless (and (equal major-mode 'planner-mode)
               (string-match planner-date-regexp (buffer-name)))
    (error "Cannot insert diary in this buffer"))
  (save-excursion
    (goto-char (point-min))
    (or (re-search-forward (concat "^" planner-diary-string "$") (point-max) t)
        (when force
          (insert planner-diary-string "\n\n\n")
          (backward-char 3)
          t)
        (error "Cannot insert diary, no diary section in this buffer"))
    ;; point is at the end of "* Diary"
    ;; (forward-char 1)
    (let ((beg (point))
          (end (if (re-search-forward "^* " (point-max) t)
                   (progn (beginning-of-line)
                          (backward-char 1)
                          (point))
                 (point-max))))
      (delete-region beg end)
      ;; point is at the end of "* Diary"
      (let ((entries (planner-diary-get-diary-entries
                      (planner-filename-to-calendar-date (buffer-name)))))
        (insert "\n\n")
        (unless (string= entries "")
          (insert entries
                  "\n"))))))


(defun planner-diary-insert-diary-maybe (&optional force)
  "Maybe insert the fancy diary for the day into the day plan file.
If the current day is in the past, don't do anything.
If FORCE is non-nil, insert `planner-diary-string' at the beginning of the
buffer if it is not there yet."
  (interactive "P")
  ;; check if we're in a day plan buffer
  (when (string-match planner-date-regexp (buffer-name))
    (if (string< (buffer-name) (planner-today))
        ;; we are in the past -> do nothing, message when interactive
        (when (interactive-p)
          (message "Date is in the past.  No diary entries inserted."))
      ;; today or future
      (planner-diary-insert-diary force))))


(defun planner-diary-show-day-plan-or-diary ()
  "Show the day plan or diary entries for the date under point in calendar.
Add this to `calendar-move-hook' if you want to use it.  In that case you
should also `remove-hook' `planner-calendar-show' from `calendar-move-hook'."
  (interactive)
  (or (planner-calendar-show)
      (view-diary-entries 1)))


(defun planner-insinuate-diary ()
  "Hook Diary into Planner.
Automatically insert and update a Diary section in day plan files.
This adds a new key binding to `planner-mode-map':
C-cC-e updates the diary sections."
  (define-key planner-mode-map "\C-c\C-e" 'planner-diary-insert-diary-maybe)
  (add-hook 'planner-goto-hook 'planner-diary-insert-diary-maybe))


(provide 'planner-diary)

;;; planner-diary.el ends here
