;;; planner-rss.el --- RSS export for the Emacs Planner (planner.el)

;; Copyright (C) 2003 Sandra Jean Chua <sacha@free.net.ph>

;; Emacs Lisp Archive Entry
;; Filename: planner-rss.el
;; Version: 2.42
;; Keywords: hypermedia
;; Author: Sacha Chua <sacha@free.net.ph>
;; Description: Export planner entries as an RSS feed
;; URL: http://sacha.free.net.ph/notebook/emacs/emacs-wiki/planner-rss.el
;; ChangeLog: http://sacha.free.net.ph/notebook/emacs/emacs-wiki/ChangeLog
;; Compatibility: Emacs20, Emacs21

;;; Commentary:
;;
;; If you use `remember-to-planner' to keep daily notes, you can
;; automatically publish remembered notes as an RSS feed by adding the
;; following code to your .emacs:
;;
;;   (add-to-list 'remember-append-to-planner-hook 'planner-rss-add-note t)
;;
;; You can also invoke `planner-rss-add-note' on any note you would like added.

;;; Code:

(require 'emacs-wiki)
(require 'planner)

(defcustom planner-rss-initial-contents
  "<?xml version=\"1.0\"?><rss version=\"2.0\"><channel>
<title>sachachua's blog</title>
<link>http://sacha.free.net.ph/notebook/wiki/today.php</link>
<description>Random notes</description>
</channel></rss>"
  "Initial contents for RSS file."
  :type 'string
  :group 'planner)

(defcustom planner-rss-base-url
  "http://sacha.free.net.ph/notebook/wiki/"
  "Base URL for blog entries. Should include trailing /."
  :type 'string
  :group 'planner)

(defcustom planner-rss-file-name
  "/home/sacha/public_html/notebook/wiki/blog.rdf"
  "Name for the main planner-rss file."
  :type 'string
  :group 'planner)

(defcustom planner-rss-category-feeds
  nil
  "List of (REGEXP FILENAME INITIAL-CONTENTS).
Blog entries that match this alist are duplicated in the given blogs.
If INITIAL-CONTENTS is non-nil, it is used to initialize the file if
the file is not found or is corrupted."
  :type '(repeat (group regexp file string)))

(defun planner-rss-initialize ()
  "Initialize the PLANNER-RSS file."
  (save-window-excursion
    (find-file planner-rss-file-name)
    (erase-buffer)
    (insert planner-rss-initial-contents)
    (save-buffer)))

(defun planner-rss-add-item (&optional title link description
                                       pubdate categories)
  "Add an item to the top of the items list in `planner-rss-file-name'.
It will have TITLE, LINK, DESCRIPTION, PUBDATE and CATEGORIES.
`planner-rss-initialize' is called if necessary."
  (save-excursion
    (save-window-excursion
      (unless (and
               (find-file planner-rss-file-name)
               (goto-char (point-min))
               (re-search-forward "<item>\\|</channel>" nil t))
        (planner-rss-initialize)
        (goto-char (point-min))
        (re-search-forward "</channel>"))
      (goto-char (match-beginning 0))
      (insert "<item>")
      (when title (insert "<title>" title "</title>"))
      (when link (insert "<link>" link "</link>"))
      (when description (insert "<description>" description "</description>"))
      (when pubdate (insert "<pubDate>" pubdate "</pubDate>"))
      (when categories (mapc 'insert categories))
      (insert "</item>")
      (save-buffer))))

(defun planner-rss-add-note ()
  "Export the current note using `planner-rss-add-item'."
  (interactive)
  (save-excursion
    (save-restriction
      (planner-narrow-to-note)
      (let ((info (planner-current-note-info t))
            (page (emacs-wiki-published-name (emacs-wiki-page-name)))
            (text (buffer-substring-no-properties (point-min) (point-max)))
            description title pubdate link category)
        ;; Get the title
        (with-temp-buffer
          (insert (elt info 2))
          (when (elt info 4)
             (insert " (" (emacs-wiki-wiki-base (elt info 4)) ")")) 
          (goto-char (point-min))
          (emacs-wiki-change-project planner-project)
          (let ((emacs-wiki-publishing-header "")
                (emacs-wiki-publishing-footer ""))
            (setq emacs-wiki-publishing-markup
                  (remove '["\\`\n*" 0 "<p>\n"] emacs-wiki-publishing-markup))
            (setq emacs-wiki-publishing-markup
                  (remove '["\\([^> \t\n]\\)\\s-*\\'" 0 "\\1\n</p>"]
                          emacs-wiki-publishing-markup))
            (emacs-wiki-replace-markup)
            (setq title (buffer-substring-no-properties (point-min)
                                                        (point-max)))))
        (with-temp-buffer
          (insert title)
          (goto-char (point-min))
          (emacs-wiki-escape-html-specials)
          (setq title (buffer-string)))
        ;; Figure out the timestamp
        ;; FIXME: Use the real time
        (setq pubdate (format-time-string "%a, %d %b %Y %H:%M:00 %z"))
        ;; Figure out the permalink
        (setq link (concat planner-rss-base-url page "#" (elt info 1)))
        ;; Figure out the category
        (when (elt info 4)
          (setq category (concat "<category>"
                                 (emacs-wiki-wiki-base (elt info 4))
                                 "</category>")))
        ;; Get the description
        (with-temp-buffer
          (insert (elt info 5))
          (goto-char (point-min))
          (emacs-wiki-change-project planner-project)
          (let ((emacs-wiki-publishing-header "")
                (emacs-wiki-publishing-footer ""))
            (emacs-wiki-replace-markup)
            (setq description (buffer-substring-no-properties (point-min)
                                                              (point-max)))))
        (with-temp-buffer
          (insert description)
          (goto-char (point-min))
          (emacs-wiki-escape-html-specials)
          (setq description (buffer-string)))
        (planner-rss-add-item title link description pubdate
                              (when category (list category)))
        (mapc (lambda (entry)
                (when (string-match (elt entry 0) text)
                  (let ((planner-rss-file-name (elt entry 1))
                        (planner-rss-initial-contents (elt entry 2)))
                    (planner-rss-add-item title link description pubdate
                                          (when category (list category))))))
              planner-rss-category-feeds)))))

(provide 'planner-rss)

;;; planner-rss.el ends here
