;;; planner-timeclock.el --- Timeclock integration for the Emacs Planner

;; Copyright (C) 2001 John Wiegley

;; Author: John Wiegley <johnw@gnu.org>
;; Maintainer: Sacha Chua <sacha@free.net.ph>
;; Keywords: planner, timeclock
;; URL: http://sacha.free.net.ph/notebook/wiki/PlannerMode.php#timeclock

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; This module allows you to clock in and clock out of your projects.
;; You can also generate reports with the <timeclock-report> tag.
;;
;;; Code:

(require 'planner)
(require 'timeclock)
(require 'planner-schedule)

(defun planner-timeclock-in ()
  "Start working on a task."
  (interactive)
  (let ((task-info (planner-current-task-info)))
    (timeclock-in nil (if (planner-task-link task-info)
                          (concat (planner-task-link task-info) ": "
                                  (planner-task-description task-info))
                        (planner-task-description task-info)))
    (planner-task-in-progress)))

(defun planner-timeclock-report-tag (beg end highlight-p)
  "Replace the region BEG to END with a timeclock report.
If HIGHLIGHT-P is non-nil, the output is also displayed."
  (require 'timeclock)
  (if highlight-p
      (add-text-properties
       beg end (list 'display
                     (with-temp-buffer
                       (timeclock-generate-report emacs-wiki-publishing-p)
                       (buffer-string))))
    (delete-region beg end)
    (timeclock-generate-report emacs-wiki-publishing-p)
    (add-text-properties beg (point) '(read-only t))))

(add-to-list 'planner-markup-tags
             '("timeclock-report" nil nil t planner-timeclock-report-tag))
(planner-option-customized 'planner-markup-tags planner-markup-tags)
(let ((map planner-mode-map))

  (define-key map "\C-c\C-i" 'planner-timeclock-in)
  (define-key map "\C-c\C-o" 'timeclock-out)
  (if (featurep 'xemacs)
      (progn
        (define-key map (kbd "C-c C-T C-i") 'planner-timeclock-in)
        (define-key map (kbd "C-c C-T C-o") 'timeclock-out))
    (define-key map (kbd "C-c C-S-t C-i") 'planner-timeclock-in)
    (define-key map (kbd "C-c C-S-t C-o") 'timeclock-out)))

(easy-menu-add-item planner-mode-map
                    '("Planner")
                    (easy-menu-create-menu
                     "Timeclock"
                     '(["Clock in" planner-timeclock-in]
                       ["Clock out" timeclock-out])))


(provide 'planner-timeclock)

;;; planner-timeclock.el ends here
