;;; planner-wl.el --- Wanderlust integration for the Emacs Planner

;; Copyright (C) 2004 Yvonne Thomson

;; Author: Yvonne Thomson (yvonne AT thewatch DOT net)
;; Version: 2.42
;; Keywords: planner, wanderlust, wl
;; URL: http://sacha.free.net.ph/notebook/wiki/PlannerMode.php

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; Add
;;
;;   (require 'planner-wl)
;;
;; to your .emacs. You will then be able to
;; call M-x planner-create-task-from-buffer from Wanderlust summary or
;; message buffers with the correct annotation.

;; To add keybindings to Wanderlust, call
;;
;;   (planner-wl-insinuate)

;; This binds F to

;;; Code:

(require 'planner)
(require 'wl)
(require 'wl-summary)

;;;###autoload
(defun planner-wl-insinuate ()
  "Hook Planner into Wanderlust.

Adds special planner keybindings to the variable `wl-summary-mode-map'.
From the Wl Summary view, you can type:

F planner-task-from-wl"
  (eval-after-load 'wl-summary
    '(define-key wl-summary-mode-map "F"
       'planner-create-task-from-buffer)))

(defun planner-wl-get-message-id ()
  "Return the message-id of the current message."
  (save-excursion
    (if     (equal major-mode 'mime-view-mode)
    (mime-preview-quit))
    (wl-summary-redisplay-all-header)
    (wl-message-select-buffer wl-message-buffer)
    (goto-char (point-min))
    (let ((case-fold-search t))
      (when (re-search-forward "^Message-ID:\\s-*\\(<.+>\\)" (point-max) t)
        (match-string 1)))))

(defun planner-wl-get-address (&optional header)
  "Return the address in HEADER of the current message.
If no HEADER is specified, \"From\" is used."
  (save-excursion
    (when (equal major-mode 'mime-view-mode)
      (mime-preview-quit))
    (wl-message-select-buffer wl-message-buffer)
    (goto-char (point-min))
    (let ((case-fold-search t))
      (when (re-search-forward
             (concat "^" (or header "From") ":\\s-*\\(.+\\)") (point-max) t)
        (match-string 1)))))

(defun planner-wl-annotation-from-wl ()
  "If called from wl, return an annotation.
Suitable for use in `planner-annotation-functions'."
  (when (equal major-mode 'wl-summary-mode)
    (let ((from (planner-wl-get-address "From")))
      (planner-make-link
       (concat "wl://" wl-summary-buffer-folder-name "/"
               (planner-wl-get-message-id))
       (concat "E-Mail "
               (if (and planner-ignored-from-addresses
                        (string-match planner-ignored-from-addresses from))
                   (concat "to " (planner-get-name-from-address
                                  (planner-wl-get-address "To")))
                 (concat "from " (planner-get-name-from-address
                                  from))))))))

(defun planner-wl-browse-url (url)
  "If this is a Wanderlust URL, jump to it."
  (when (string-match "^wl://\\(.+\\)/\\(.+\\)" url)
    (let ((group (match-string 1 url))
          (article (match-string 2 url)))
      (wl-folder-goto-folder-subr group)
      (wl-folder-jump-to-current-entity)
      (wl-summary-jump-to-msg-by-message-id article))))

(defun planner-wl-resolve-url (id)
  "A Wanderlust ID should not be displayed."
  "")

(fset 'planner-get-from 'planner-wl-get-address)
(fset 'planner-get-message-id 'planner-wl-get-message-id)
(custom-add-option 'planner-annotation-functions
                   'planner-wl-annotation-from-wl)
(add-hook 'planner-annotation-functions 'planner-wl-annotation-from-wl)
(add-hook 'planner-browse-url-functions 'planner-wl-browse-url)
(planner-option-customized 'planner-url-list
                           (append (list "wl://") planner-url-list))
(add-to-list 'planner-resolve-url-table
             '("wl://" . planner-gnus-resolve-url))

(provide 'planner-wl)

;;; planner-wl.el ends here
