;;; remember-bbdb --- BBDB support for remember.el

;; Copyright (C) 1999, 2000, 2001 John Wiegley
;; Copyright (C) 2003 Sandra Jean Chua

;; Author: John Wiegley <johnw@gnu.org>
;; Maintainer: Sacha Chua <sacha@free.net.ph>
;; Created: 29 Mar 1999
;; Version: 2.42
;; Keywords: data memory todo pim bbdb
;; URL: http://sacha.free.net.ph/notebook/emacs/emacs-wiki/remember-bbdb.el

;; This file is not part of GNU Emacs.

;; This is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2, or (at your option) any later
;; version.
;;
;; This is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
;; MA 02111-1307, USA.

;;; Commentary:

;; Remembering to a planner.el task page
;; Requires at least planner 1.197, available from
;; http://sacha.free.net.ph/notebook/emacs/emacs-wiki/planner.el

;;; Code:

(require 'planner)
(require 'remember)

(defcustom remember-planner-copy-on-xref-flag t
  "*Non-nil means copy note text instead of moving it to the plan page."
  :type 'boolean
  :group 'remember)

(defvar remember-planner-timestamp-format " %H:%M"
  "*Format of timestamp for remember entries.
If you don't want timestamps, just set this to \"\".")

(defun remember-append-to-planner (&optional page)
  "Remember, with description DESC, the given TEXT."
  (require 'planner)
  (set-buffer (get-buffer-create remember-buffer))
  (let ((text (buffer-string))
        start)
    (save-window-excursion
      (planner-create-note page)
      (setq start (line-beginning-position))
      (insert text)
      (save-restriction
        (narrow-to-region start (point))
        (run-hooks 'remember-append-to-planner-hook))
      (when remember-save-after-remembering
        (save-restriction (save-buffer))))))

(defvar remember-append-to-planner-hook '(remember-planner-add-timestamp
                                          remember-planner-add-xref)
  "Functions to run after something has been appended to the planner day page.
Buffer will be narrowed to the current note.")

(defun remember-planner-add-timestamp ()
  "Add a timestamp to the current entry.
This function can be added to `remember-append-to-planner-hook'."
  (goto-char (point-min))
  (goto-char (line-end-position))
  (insert (format-time-string remember-planner-timestamp-format
                              (current-time))))

(defvar remember-planner-page nil
  "Planner page to save to by default.")

(defun remember-planner-add-xref ()
  "Move the main text into a plan page.
Replace the day page entry with a cross-reference."
  (goto-char (point-min))
  (when (looking-at "^.#\\([0-9]+\\)\\s-+\\(.*\\)")
    (let* (plan-number
           (day-number (match-string-no-properties 1))
           (day-page (planner-today))
           (title (match-string-no-properties 2))
           (emacs-wiki-default-page (or remember-planner-page (planner-today)))
           (body (buffer-substring-no-properties (line-end-position)
                                                 (point-max)))
           (plan-page (with-emacs-wiki-project planner-project
                        (emacs-wiki-read-name (emacs-wiki-file-alist)))))
      (unless (equal plan-page (planner-today))
        (save-window-excursion
          (setq plan-number (number-to-string (planner-create-note plan-page)))
          (insert title " ([[" day-page "#" day-number "]])" body)
          (when remember-save-after-remembering
            (save-restriction (save-buffer))))
        (if remember-planner-copy-on-xref-flag
            (goto-char (line-end-position))
          (delete-region (line-end-position) (point-max))
          (goto-char (point-max)))
        (insert " ([[" plan-page "#" plan-number "]])")))))

;; Old functions collapsed
(fset 'remember-append-to-planner-plan-page 'remember-append-to-planner)
(fset 'remember-to-planner-plan-page-dwim 'remember-to-planner)
(fset 'remember-to-planner-plan-page 'remember-to-planner)

(defvar remember-planner-page nil "Default planner page.")

(defun remember-to-planner ()
  "Call `remember' and save the results to a plan page.
Defaults to saving to the current planner page if the current
buffer is a plan (not day) page.

Planner.el users should use this instead of `remember' because
this saves more context information."
  (interactive)
  (let ((page (and (equal major-mode 'planner-mode)
                   (not (string-match planner-date-regexp
                                      (emacs-wiki-page-name)))
                   (emacs-wiki-page-name))))
    (remember)
    (set-buffer (get-buffer-create remember-buffer))
    (set (make-variable-buffer-local 'remember-handler-functions)
         'remember-append-to-planner-plan-page)
    (setq remember-planner-page page)))

(custom-add-option 'remember-handler-functions 'remember-append-to-planner)


(provide 'remember-planner)

;;; remember-planner.el ends here
